@@ -4,28 +4,31 @@ use strict;
 
 use vars qw($VERSION);
 
-$VERSION = 0.07;
+$VERSION = '0.10';
 
 use Class::AlzaboWrapper::Cursor;
 
 use Exception::Class ( 'Class::AlzaboWrapper::Exception::Params' );
 Class::AlzaboWrapper::Exception::Params->Trace(1);
 
-use Params::Validate qw( validate validate_with SCALAR UNDEF ARRAYREF HASHREF );
+use Params::Validate qw( validate validate_pos validate_with SCALAR UNDEF ARRAYREF HASHREF );
 Params::Validate::validation_options
     ( on_fail =>
       sub { Class::AlzaboWrapper::Exception::Params->throw
                 ( message => join '', @_ ) } );
 
 my %TableToClass;
+my %ClassToTable;
 my %ClassAttributes;
 
 BEGIN
 {
     foreach my $sub ( qw( select update delete is_live ) )
     {
+        my $sub = sub { shift->row_object->$sub(@_) };
+
         no strict 'refs';
-        *{ __PACKAGE__ ."::$sub" } = sub { shift->row_object->$sub(@_) };
+        *{ __PACKAGE__ ."::$sub" } = $sub;
     }
 }
 
@@ -48,10 +51,9 @@ sub import
                      );
 
     my $base = delete $p{base};
+    eval "package $p{caller}; use base '$base'";
 
     $class->_make_methods(%p);
-
-    eval "package $p{caller}; use base '$base'";
 }
 
 sub _make_methods
@@ -59,32 +61,67 @@ sub _make_methods
     my $class = shift;
 
     my %p = validate( @_,
-                      { skip   => { type => ARRAYREF, default => [] },
+                      { skip   => { type => SCALAR | ARRAYREF, default => [] },
                         table  => { isa => 'Alzabo::Table' },
-                        caller => { type => SCALAR },
+                        caller => { type => SCALAR, default => $class },
                       }
                     );
 
-    my %skip = map { $_ => 1 } @{ $p{skip} };
-    foreach my $name ( map { $_->name } $p{table}->columns )
+    my $caller = delete $p{caller};
+
+    $caller->SetAlzaboTable( delete $p{table} );
+    $caller->MakeColumnMethods(%p);
+}
+
+sub SetAlzaboTable
+{
+    my $class = shift;
+    my ($table) = validate_pos( @_, { isa => 'Alzabo::Table' } );
+
+    $TableToClass{ $table->name } = $class;
+    $ClassToTable{$class} = $table;
+}
+
+sub Table
+{
+    my $class = ref $_[0] || $_[0];
+
+    die "Must call SetTable() before calling Table()"
+         unless $ClassToTable{$class};
+
+    return $ClassToTable{$class};
+
+}
+# deprecated
+*table = \&Table;
+
+sub MakeColumnMethods
+{
+    my $class = shift;
+    my %p = validate( @_,
+                      { skip   => { type => SCALAR | ARRAYREF, default => [] },
+                      }
+                    );
+
+    my %skip = map { $_ => 1 } ref $p{skip} ? @{ $p{skip} } : $p{skip};
+
+    my $table = $class->Table;
+    foreach my $name ( map { $_->name } $table->columns )
     {
         next if $skip{$name};
 
+        $class->_RecordAttributeCreation( $class => $name );
+
+        my $sub = sub { shift->row_object->select($name) };
+
         no strict 'refs';
-        *{"$p{caller}\::$name"} = sub { shift->row_object->select($name) };
-
-        $class->_record_attribute_creation( $p{caller} => $name );
+        *{"$class\::$name"} = $sub;
     }
-
-    {
-        no strict 'refs';
-        *{"$p{caller}\::table"} = sub { $p{table} };
-    }
-
-    $TableToClass{ $p{table}->name } = $p{caller};
 }
 
-sub _record_attribute_creation { push @{ $ClassAttributes{ $_[1] } }, $_[2] }
+sub _RecordAttributeCreation { push @{ $ClassAttributes{ $_[1] } }, $_[2] }
+# deprecated
+*record_attribute_creation = \&_RecordAttributeCreation;
 
 sub new
 {
@@ -137,8 +174,6 @@ sub new
     return $self;
 }
 
-sub params_exception { shift; die @_ }
-
 sub create
 {
     my $class = shift;
@@ -166,10 +201,13 @@ sub potential
         $class->new( object => $class->table->potential_row( values => {@_} ) );
 }
 
-sub columns { shift->table->columns(@_) }
-*column = \&columns;
+sub Columns { shift->table->columns(@_) }
+*Column = \&Columns;
+# deprecated
+*columns = \&Columns;
+*column = \&Columns;
 
-sub cursor
+sub Cursor
 {
     my $self = shift;
 
@@ -177,17 +215,23 @@ sub cursor
         Class::AlzaboWrapper::Cursor->new
             ( cursor => shift );
 }
+# deprecated
+*cursor = \&Cursor;
 
-sub row_object { $_[0]->{row} }
+sub TableToClass { $TableToClass{ $_[1]->name } }
+# deprecated
+*table_to_class = \&TableToClass;
 
-sub table_to_class { $TableToClass{ $_[1]->name } }
-
-sub alzabo_attributes
+sub AlzaboAttributes
 {
     my $class = ref $_[0] || $_[0];
 
     @{ $ClassAttributes{$class} };
 }
+# deprecated
+*alzabo_attributes = \&AlzaboAttributes;
+
+sub row_object { $_[0]->{row} }
 
 
 1;
@@ -200,13 +244,20 @@ Class::AlzaboWrapper - Higher level wrapper around Alzabo Row and Table objects
 
 =head1 SYNOPSIS
 
-  use Class::AlzaboWrapper ( table => $schema->table('User') );
+  package WebTalk::User;
+  use base 'Class::AlzaboWrapper';
+
+  __PACKAGE->SetAlzaboTable( $schema->table('User') );
+  __PACKAGE->MakeColumnMethods();
 
 =head1 DESCRIPTION
 
 This module is intended for use as a base class when you are writing
 a class that wraps Alzabo's table and row classes.
 
+It also provides a way to generate some methods specific to your
+subclass.
+
 =head1 USAGE
 
 Our usage examples will assume that there is database containing
@@ -219,26 +270,10 @@ This module throws exceptions when invalid parameters are given to
 methods.  The exceptions it throws are objects which inherit from
 C<Exception::Class::Base>, just as with Alzabo itself.
 
-=head2 Import
+=head2 SetAlzaboTable()
 
-When a subclass imports this module, it should pass a "table"
-parameter to it, which should be an C<Alzabo::Runtime::Table> object.
-This is considered the "main table" being wrapped by the given
-subclass, though it can of course access other tables freely.
-
-So for our hypothetical C<WebTalk::User> class, we would pass the
-"User" table when importing C<Class::AlzaboWrapper>.
-
-When importing the module, you can also pass a "skip" method, which
-should be an array reference.  This reference contains the names of
-columns for which methods should not be auto-generated.  See the
-L<Generated methods section|/Generated methods> below for more details.
-
-When you import the module, it will make sure that your class is
-declared as a subclass of C<Class::AlzaboWrapper> automatically.
-
-If invalid parameters are given when importing the module, it will
-throw a C<Class::AlzaboWrapper::Exception::Params> exception.
+This method must be called by your subclass or almost none of the
+methods provided by C<Class::AlzaboWrapper> will work.
 
 =head2 Inherited methods
 
@@ -248,7 +283,7 @@ Subclasses inherit a number of method from C<Class::AlzaboWrapper>.
 
 =over 4
 
-=item * new
+=item * new(...)
 
 The C<new()> method provided allows you to create new objects either
 from an Alzabo row object, or from the main table's primary keys.
@@ -271,41 +306,41 @@ C<new()> method to the caller.
 If invalid parameters are given then this method will throw a
 C<Class::AlzaboWrapper::Exception::Params> exception.
 
-=item * create
+=item * create(...)
 
 This method is used to create a new object and insert it into the
 database.  It simply calls the C<insert()> method on the class's
 associated table object.  Any parameters given to this method are
 passed given to the C<insert()> method as its "values" parameter.
 
-=item * potential
+=item * potential(...)
 
 This creates a new object based on a potential row, as opposed to one
 in the database.  Similar to the C<create()> method, any parameters
 passed are given to the table's C<potential_row()> method as the
 "values" parameter.
 
-=item * columns
+=item * Columns(...)
 
 This is simply a shortcut to the associated table's C<columns> method.
 This may also be called as an object method.
 
-=item * column
+=item * Column(...)
 
 This is simply a shortcut to the associated table's C<column> method.
 This may also be called as an object method.
 
-=item * table
+=item * Table()
 
 This method returns the Alzabo table object associated with the
 subclass.  This may also be called as an object method.
 
-=item * alzabo_attributes
+=item * AlzaboAttributes()
 
 Returns a list of accessor methods that were created based on the
 columns in the class's associated table.
 
-=item * cursor ($cursor)
+=item * Cursor ($cursor)
 
 Given an C<Alzabo::Runtime::Cursor> object (either a row or join
 cursor), this method returns a new C<Class::AlzaboWrapper::Cursor>
@@ -317,31 +352,34 @@ object.
 
 =over 4
 
-=item * row_object
+=item * row_object()
 
 This method returns the C<Alzabo::Runtime::Row> object associated with
 the given subclass object.  So, for our hypothetical C<WebTalk::User>
 class, this would return an object representing the underlying row
 from the User table.
 
-=item * select / update / delete / is_live
+=item * select() / update() / delete() / is_live()
 
-These methods are simply passthroughs to the underlying Alzabo row's
+These methods are simply passthroughs to the underlying Alzabo row
 methods of the same names.  You may want to subclass some of these in
 order to change their behavior.
 
 =back
 
-=head3 Generated methods
+=head3 MakeColumnMethods(...)
 
-For each column in the associated table, their is a method created
-that selects that column's value from the underlying row for an
-object.  For example, if our User table contained "username" and
-"email" columns, then our C<WebTalk::User> object would have
-C<username()> and C<email()> methods generated.
+If you call this method on your subclass, then for each column in the
+associated table, a method will be created in your subclass that
+selects that column's value from the underlying row for an object.
 
-As was mentioned before, if a column is listed in the "skip" parameter
-when this module is imported, this method will not be made.
+For example, if our User table contained "username" and "email"
+columns, then our C<WebTalk::User> object would have C<username()> and
+C<email()> methods generated.
+
+The C<MakeColumnMethods()> method accepts a "skip" parameter which can
+be either a scalar or array reference.  This is a list of columns for
+which methods I<should not> be generated.
 
 =head3 Class::AlzaboWrapper methods
 
@@ -349,7 +387,7 @@ The C<Class::AlzaboWrapper> module has a method it provides:
 
 =over 4
 
-=item * table_to_class ($table)
+=item * TableToClass($table)
 
 Given an Alzabo table object, this method returns its associated
 subclass.
@@ -361,46 +399,15 @@ subclass.
 When using this module, you need to use the
 C<Class::AlzaboWrapper::Cursor> module to wrap Alzabo's cursor
 objects, so that objects the cursor returns are of the appropriate
-subclass, not plain C<Alzabo::Runtime::Row> objects.
-
-=head2 Subclassing
-
-If you want to subclass this module, you may want to override the
-C<import()> method in order to do something like create methods in the
-calling class.  If you do this, you should call Class::AlzaboWrapper's
-C<import()> method as well.  You'll need to override the "base" and
-"caller" parameters when doing this.  Set "base" to your subclass and
-"caller" to the class that called your import method.  Here is an
-example:
-
-  package My::AlzaboWrapper;
-
-  use base 'Class::AlzaboWrapper';
-
-  sub import
-  {
-      my $class = shift;
-
-      # called via use base
-      return unless @_;
-
-      my %p = @_;
-
-      my $caller = (caller(0))[0];
-
-      $class->SUPER::import( %p,
-                             base   => $class,
-                             caller => $caller,
-                           );
-
-      $class->_make_more_methods(%p);
-  }
+subclass, not plain C<Alzabo::Runtime::Row> objects.  The C<Cursor()>
+method provides some syntactic sugar for creating
+C<Class::AlzaboWrapper::Cursor> objects.
 
 =head3 Attributes created by subclasses
 
 If you want to record the accessor methods your subclass makes so they
-are available via C<alzabo_attributes()>, you can call the
-C<_record_attribute_creation()> method, which expects two arguments.
+are available via C<AlzaboAttributes()>, you can call the
+C<_RecordAttributeCreation()> method, which expects two arguments.
 The first argument is the class for which the method was created and
 the second is the name of the method.
 
@@ -417,13 +424,12 @@ see your questions.
 
 =head1 SEE ALSO
 
-The Regional Vegetarian Guide at http://www.regveg.org/ is a site I
-created which actually uses this code as part of the application.  Its
-source is available from the web site.
+VegGuide.Org is a site I created which actually uses this code as part
+of the application.  Its source is available from the web site.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2002-2003 David Rolsky.  All rights reserved.  This
+Copyright (c) 2002-2005 David Rolsky.  All rights reserved.  This
 program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
